local _G = getfenv(0)
local ADDON_NAME, addon = ...

local ipairs = _G.ipairs
local pairs = _G.pairs
local floor = _G.math.floor
local select = _G.select
local tostring = _G.tostring
local tsort = _G.table.sort
local tinsert = _G.table.insert

local SpellNames = addon.SpellNames
local ItemNames = addon.ItemNames

local CompactRunes = LibStub("AceAddon-3.0"):GetAddon(addon.addonName)

function CompactRunes:PerformMigrations()
  self:MigrateSettings()
  self:MigrateFreezingFogToRime()
  self:MigrateBoneShieldLow()
  self:MigrateShadowInfusion()
end

function CompactRunes:MigrateShadowInfusion()
  -- Shadow Infusion was removed and is no longer a necessary option
  -- for the Dark Transformation bar.  If set for Show on Cooldown, change
  -- to "Never".  Should be the closest option.
  if self.db.profile.bars.DarkTransformationBar.showCooldown == "ShadowInfusion" then
    self.db.profile.bars.DarkTransformationBar.showCooldown = "Never"
  end
end

function CompactRunes:MigrateFreezingFogToRime()
	-- Change the old Freezing Fog to Rime so old profiles keep
	-- the proper ordering.
	for i, v in ipairs(self.db.profile.priorityList["Frost"]) do
		if v == "Freezing Fog" then
			self.db.profile.priorityList["Frost"][i] = "Rime"
		end
	end
end

function CompactRunes:MigrateBoneShieldLow()
  -- Change the old hard-coded 'Bone Shield <= 6' proc to the new one.
	for i, v in ipairs(self.db.profile.priorityList["Blood"]) do
		if v == "Bone Shield <= 6" then
			self.db.profile.priorityList["Blood"][i] = "Bone Shield <= X"
		end
	end
end

function CompactRunes:MigrateSettings()
	local version = self.db.profile.profileVersion
	if version and version < 2 then
		for spec, list in pairs(self.db.profile.priorityList) do
			self.db.profile.priorityList[spec] = {}
		end
	end
	self.db.profile.profileVersion = 2

	if self.db.profile.runeRechargeNums ~= nil then
		if self.db.profile.runeRechargeNums then
			self.db.profile.rechargeTimers = "All"
		end
		self.db.profile.runeRechargeNums = nil
	end
	if self.db.profile.width ~= nil then
		self.db.profile.dimensions["HORIZONTAL"].width = self.db.profile.width
		self.db.profile.width = nil
	end
	if self.db.profile.height ~= nil then
		self.db.profile.dimensions["HORIZONTAL"].height = self.db.profile.height
		self.db.profile.height = nil
	end
	if self.db.profile.spacing ~= nil then
		self.db.profile.dimensions["HORIZONTAL"].spacing = self.db.profile.spacing
		self.db.profile.spacing = nil
	end
	if self.db.profile.runes and self.db.profile.runes.inset ~= nil then
		self.db.profile.runes.inset = nil
		self.db.profile.runes = nil
	end
	if self.db.profile.bars.DarkTransformationBar and
		self.db.profile.bars.DarkTransformationBar.shadowInfusion ~= nil then
		if self.db.profile.bars.DarkTransformationBar.shadowInfusion == false then
			self.db.profile.bars.DarkTransformationBar.showCooldown = "Never"
		end
		self.db.profile.bars.DarkTransformationBar.shadowInfusion = nil
	end
end
